#pragma semicolon 1
#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <dhooks>
#include <vip_core>

#pragma newdecls required

public Plugin myinfo = 
{
	name = "[VIP] Extended Ammo",
	author = "R1KO",
	version = "1.10"
};

static const char g_sFeature[][] = {"ExtendedAmmoClip", "ExtendedAmmoReserve"};

public void VIP_OnVIPLoaded()
{
	VIP_RegisterFeature(g_sFeature[0], INT, _, _, OnItemDisplay);
	VIP_RegisterFeature(g_sFeature[1], INT, _, _, OnItemDisplay);
}

public void OnPluginEnd() 
{
	if(CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "VIP_UnregisterFeature") == FeatureStatus_Available)
	{
		VIP_UnregisterFeature(g_sFeature[0]);
		VIP_UnregisterFeature(g_sFeature[1]);
	}
}

public bool OnItemDisplay(int iClient, const char[] sFeatureName, char[] sDisplay, int iMaxLen)
{
	if(VIP_IsClientFeatureUse(iClient, sFeatureName))
	{
		SetGlobalTransTarget(iClient);
		FormatEx(sDisplay, iMaxLen, "%t [+ %i %%]", sFeatureName, VIP_GetClientFeatureInt(iClient, sFeatureName));
		return true;
	}

	return false;
}

StringMap g_hWeaponsAmmoClipTrie;
StringMap g_hWeaponsAmmoReserveTrie;

Handle hGetMaxClip1;
Handle hGetReserveAmmoMax;

public void OnPluginStart() 
{
	Handle hGameConfig = LoadGameConfigFile("dhooks_vip_ext_ammo.games");
	
	if(hGameConfig == INVALID_HANDLE)
	{
		SetFailState("Invlid Game Data");
	}

	hGetMaxClip1 = DHookCreate(GameConfGetOffset(hGameConfig, "GetWeaponClip1"), HookType_Entity, ReturnType_Int, ThisPointer_CBaseEntity, DHOnGetMaxClip1);
	hGetReserveAmmoMax = DHookCreate(GameConfGetOffset(hGameConfig, "GetWeaponReserve"), HookType_Entity, ReturnType_Int, ThisPointer_CBaseEntity, DHOnGetReserveAmmoMax);
	DHookAddParam(hGetReserveAmmoMax, HookParamType_Unknown);

	g_hWeaponsAmmoClipTrie = new StringMap();
	g_hWeaponsAmmoReserveTrie = new StringMap();
	
	LoadTranslations("vip_modules.phrases");
	
	if(VIP_IsVIPLoaded())
	{
		VIP_OnVIPLoaded();
	}
}

public void OnMapStart()
{
	g_hWeaponsAmmoClipTrie.Clear();
	g_hWeaponsAmmoReserveTrie.Clear();

	char sBuffer[256];
	KeyValues hKeyValues;

	hKeyValues = new KeyValues("ExtendedAmmo");
	BuildPath(Path_SM, sBuffer, sizeof(sBuffer), "data/vip/modules/extended_ammo.ini");

	if (hKeyValues.ImportFromFile(sBuffer) == false)
	{
		delete hKeyValues;
		SetFailState("Не удалось открыть файл \"%s\"", sBuffer);
	}

	hKeyValues.Rewind();

	if(hKeyValues.GotoFirstSubKey())
	{
		do
		{
			hKeyValues.GetSectionName(sBuffer, 64);
			g_hWeaponsAmmoClipTrie.SetValue(sBuffer, hKeyValues.GetNum("clip"));
			g_hWeaponsAmmoReserveTrie.SetValue(sBuffer, hKeyValues.GetNum("reserve"));
		}
		while (hKeyValues.GotoNextKey());
	}

	delete hKeyValues;
}

public void OnEntityCreated(int iEntity, const char[] sClassName)
{
	//	LogMessage("OnEntityCreated: %i, %s", iEntity, sClassName);
	if(strncmp(sClassName, "weapon_", 7) == 0)
	{
		SDKHook(iEntity, SDKHook_SpawnPost, OnWeaponSpawn);
	}
}

public void OnWeaponSpawn(int iEntity)
{
	SDKUnhook(iEntity, SDKHook_SpawnPost, OnWeaponSpawn);

	//	LogMessage("OnWeaponSpawn: %i", iEntity);
	DHookEntity(hGetMaxClip1, true, iEntity);
	DHookEntity(hGetReserveAmmoMax, true, iEntity);
}

public MRESReturn DHOnGetMaxClip1(int iEntity, Handle hReturn)
{
	int iClient = GetEntPropEnt(iEntity, Prop_Data, "m_hOwner");
	//	LogMessage("DHOnGetMaxClip1: %i -> m_hOwner: %i", iEntity, iClient);
	if(iClient != -1 && VIP_IsClientVIP(iClient) && VIP_IsClientFeatureUse(iClient, g_sFeature[0]))
	{
		//	int MaxClip = DHookGetReturn(hReturn);
		//	LogMessage("MaxClip: %i", MaxClip);
		
		char sClassName[64];
		int iAmmo;
		GetEntityClassname(iEntity, sClassName, sizeof(sClassName));

		if(g_hWeaponsAmmoClipTrie.GetValue(sClassName, iAmmo))
		{
			//	LogMessage("iAmmo: %i, vip: %i = %i", iAmmo, VIP_GetClientFeatureInt(iClient, g_sFeature[0]), iAmmo+RoundToCeil((float(iAmmo)*float(VIP_GetClientFeatureInt(iClient, g_sFeature[0])))/100.0));

			DHookSetReturn(hReturn, iAmmo+RoundToCeil((float(iAmmo)*float(VIP_GetClientFeatureInt(iClient, g_sFeature[0])))/100.0));
			return MRES_Override;
		}
	}

	return MRES_Ignored;
}

public MRESReturn DHOnGetReserveAmmoMax(int iEntity, Handle hReturn)
{
	int iClient = GetEntPropEnt(iEntity, Prop_Data, "m_hOwner");
	//	LogMessage("DHOnGetReserveAmmoMax: %i -> m_hOwner: %i", iEntity, iClient);
	if(iClient != -1 && VIP_IsClientVIP(iClient) && VIP_IsClientFeatureUse(iClient, g_sFeature[1]))
	{
		char sClassName[64];
		GetEntityClassname(iEntity, sClassName, sizeof(sClassName));
		int iAmmo;
		if(g_hWeaponsAmmoReserveTrie.GetValue(sClassName, iAmmo))
		{
			//	LogMessage("iAmmo: %i, vip: %i = %i", iAmmo, VIP_GetClientFeatureInt(iClient, g_sFeature[1]), iAmmo+RoundToCeil((float(iAmmo)*float(VIP_GetClientFeatureInt(iClient, g_sFeature[1])))/100.0));

			DHookSetReturn(hReturn, iAmmo+RoundToCeil((float(iAmmo)*float(VIP_GetClientFeatureInt(iClient, g_sFeature[1])))/100.0));
			return MRES_Override;
		}
	}

	return MRES_Ignored;
}